/*  QMC: simplification tool (by using the Quine - McClusky process)
    Copyright (C) 2000  Thomas Pollak

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  

Auf Deutsch:

    QMC: Vereinfachungsprogramm fuer logische Terme nach Quine - McClusky
    Copyright (C) 2000  Thomas Pollak

    Dieses Programm ist freie Software. Sie koennen es unter
    den Bedingungen der GNU General Public License, wie von der
    Free Software Foundation herausgegeben, weitergeben und/oder
    modifizieren, entweder unter Version 2 der Lizenz oder
    jeder spaeteren Version.

    Die Veroeffentlichung dieses Programms erfolgt in der
    Hoffnung, dass es Ihnen von Nutzen sein wird, aber OHNE JEDE
    GEWAEHRLEISTUNG - sogar ohne die implizite Gewaehrleistung
    der MARKTREIFE oder der EIGNUNG FUER EINEN BESTIMMTEN ZWECK.
    Details finden Sie in der GNU General Public License.

    Sie sollten eine Kopie der GNU General Public License zusammen
    mit diesem Programm erhalten haben. Falls nicht, schreiben Sie
    an die Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
    MA 02139, USA.
*/

//Headers
#include <cqmc>
#include <cstring>


/************
 *          *
 * brackets *
 *          *
 ************/


//Debug - Switch
//#define DEBUG

#ifdef DEBUG
  #include <iostream>
#endif

using namespace std;

unsigned char klammern(char *term, unsigned char &anz, char *variablen)
{

  //local variables
  char ks=0;
  unsigned char j,i,swap=0;
  anz=0;


  //initialize array
  for(j=0;j<VAR_ANZ;j++)
  {
    variablen[j]='#';
  }

  //close array with zero
  variablen[j]='\0';


  #ifdef DEBUG
    cerr << "\nKLAMMERN init vars: " << variablen;
  #endif


  //MAINLOOP - step trough the boolean term
  for(i=0;i<strlen(term)+1;i++)
  {

    //check current character
    if(term[i]==SYN_KLAMMER_AUF)
    {
      ks++;
    }
    else if(term[i]==SYN_KLAMMER_ZU)
    {
      ks--;
    }
    else if( (term[i]==SYN_UND) || (term[i]==SYN_ODER) )
    {
      if( swap )
      {
        
         #ifdef DEBUG
           cerr << "\nKLAMMERN bad order of control char!";
         #endif
         
        return(4);
      }
      
      swap=1;
    }
    else if( term[i]==SYN_NICHT )
    {
      if( swap==2 )
      {

         #ifdef DEBUG
           cerr << "\nKLAMMERN bad order of not char!";
         #endif
         
        return(5);
      }
      
      swap=2;
    }
    else if( (term[i]!=SYN_FALSCH) || (term[i]!=SYN_WAHR) )
    {
      //create new object if it isn't existing or if it isn't a control char
      //and increase the number of vars
      j=0;

      while(variablen[j]!=term[i])
      {
        j++;

        if(j>VAR_ANZ)
        {
          j=0;

          while(variablen[j]!='#')
          {
            j++;

            //end function if the maximum of variables is reached
            if(j>VAR_ANZ)
              return(3);
          }

          variablen[j]=term[i];
          anz++;

          #ifdef DEBUG
            cerr << "\nKLAMMERN runtime variables: " << variablen << " number: " << (unsigned short)anz;
          #endif
        }
      }
    
      
      if( swap ) swap=0;
       
    }

    //end function if there is a mistake with the brackets
    if(ks<0)
    {
       //delete [] variablen;

       #ifdef DEBUG
         cerr << "\nKLAMMERN negative result of brackets!";
       #endif

       return(2);
    }
  }


  //end function if not zero
  if(ks!=0)
  {
    #ifdef DEBUG
      cerr << "\nKLAMMERN result is not zero! (ks!=0)";
    #endif

    return(1);
  }

  //end function if there is no variable
  if(anz==0)
  {
    #ifdef DEBUG
      cerr << "\nKLAMMERN no variable found!";
    #endif
    
    return(1);
  }


  //send OK to the main function
  return(0);
}

